from __future__ import print_function
import glob
import itertools
import optparse
import os
import os.path
import PIL.Image as Image
import re
import scipy as sp

# Parse options
parser = optparse.OptionParser(usage="Usage: %prog [options] basenames", description="Gather statistics on noise reduction performance.")
parser.add_option('--sigma', action="append", type=str, default=[], help="Standard deviation(s) of noise (in the form present in the filename).")
parser.add_option('--rho', action="append", type=str, default=[], help="Correlation factor(s) of noise.")
parser.add_option('--space', action="append", type=str, default=[], help="Space(s) used for filtering.")
parser.add_option('--op', action="append", type=str, default=[], help="Operation(s) used for filtering.")
(options, args) = parser.parse_args()
if len(args) == 0:
    parser.error("At least one base name required.")

basenames = args
sigmas = options.sigma if len(options.sigma)>0 else ["32"]
rhos = options.rho if len(options.rho)>0 else ["0"]
spaces = options.space if len(options.space)>0 else ["basis","circle","sphere","lexicographical"]
operationNames = options.op if len(options.op)>0 else ["occo-3x3-cross"]

with open("noiseResults.csv","wt") as out:
    out.write('"operationName","basename","sigma","rho"')
    for space in spaces:
        #out.write(',"%s (min nmse)"' % space)
        out.write(',"%s (mean nmse)"' % space)
        #out.write(',"%s (max nmse)"' % space)
        #out.write(',"%s (min rmse)"' % space)
        out.write(',"%s (mean rmse)"' % space)
        #out.write(',"%s (max rmse)"' % space)
        #out.write(',"%s (min cos)"' % space)
        #out.write(',"%s (mean cos)"' % space)
        #out.write(',"%s (max cos)"' % space)
    out.write("\n")

    for (operationName,sigmaText,rho,basename) in itertools.product(operationNames,sigmas,rhos,basenames):
        out.write("%s,%s,%s,%s" % (operationName,basename,sigmaText,rho))
        Iorg = sp.array(Image.open("../Images/%s.png" % basename),dtype="float32")
        IorgSumSquared = sp.power(sp.absolute(Iorg),2).sum()
        if Iorg.shape[2]>3:
            Iorg = Iorg[:,:,:3]
        elif Iorg.shape[2]!=3:
            print("Invalid (non-RGB) image: %s" % basename)
            continue
        rowName = "%s-sigma=%s-rho=%s" % (basename,sigmaText,rho)
        print("Processing: %s, %s, %s, %s" % (operationName,basename,sigmaText,rho))
        for space in spaces:
            nmses = []
            rmses = []
            #coses = []
            for noisyImage in glob.glob("../Images/noisy/%s-[0-9][0-9][0-9].png" % rowName):
                noisyImageBase = re.match(".*/noisy/(.+)\.png$", noisyImage).group(1)
                newImageName = "out/%s-%s/noisy/%s.png" % (operationName,space,noisyImageBase)
                InoisySquaredError = sp.power(  sp.absolute(sp.array(Image.open(noisyImage),dtype="float32")  -Iorg),2).sum()
                InewSquaredError = sp.power(  sp.absolute(sp.array(Image.open(newImageName),dtype="float32")  -Iorg),2).sum()
                nmses.append(InewSquaredError/IorgSumSquared)
                rmses.append(InewSquaredError/InoisySquaredError)
                #coses.append(sp.dot((Inew-Inoisy).flat,(Iorg-Inoisy).flat)/sp.sqrt(sp.power(sp.absolute(Inew-Inoisy),2).sum()*sp.power(sp.absolute(Iorg-Inoisy),2).sum()))
            #out.write(",%.6g,%.6g,%.6g,%.6g,%.6g,%.6g,%.6g,%.6g,%.6g" % (min(nmses),sp.mean(nmses),max(nmses),min(rmses),sp.mean(rmses),max(rmses),min(coses),sp.mean(coses),max(coses)))
            #out.write(",%.6g,%.6g,%.6g" % (min(nmses),sp.mean(nmses),max(nmses)))
            #out.write(",%.6g,%.6g,%.6g" % (min(rmses),sp.mean(rmses),max(rmses)))
            out.write(",%.6g" % sp.mean(nmses))
            out.write(",%.6g" % sp.sqrt(sp.mean(rmses)))
        out.write('\n')

