from __future__ import print_function
from operations import operations
from optparse import OptionParser
import itertools
import os
import PIL.Image as Image
import re
import scipy as sp
import scipy.linalg as la

parser = OptionParser(usage="Usage: %prog [options] image")
parser.add_option("-o", "--operation", type="choice", default="occo-3x3-cross", choices=list(iter(operations(1).keys())))
parser.add_option("-s", "--space", type=str, default="sphere")
parser.add_option("--maxn", type=int, default=150)
parser.add_option("--scale", type=int, default=1)
parser.add_option("--intermediates", action="store_true", default=False)
parser.add_option("--out", type=str, default=None)
(options,args) = parser.parse_args()
if len(args) != 1:
    parser.error("Exactly one image required.")

try:
    # The __import__ statement is a little weird, when importing a submodule, it only returns the submodule (instead of the top-level module) if fromlist is given (and non-empty).
    space = __import__("spaces.%s" % options.space, fromlist=[''])
except ImportError:
    parser.error("Unsupported space: %s." % options.space)

inputFilename = args[0]
operation = operations(options.scale)[options.operation]
saveIntermediates = options.intermediates

if space.n:
    maxn = space.n
else:
    maxn = options.maxn

if options.out is None:
    outputFilename = '%s-out.png' % os.path.splitext(inputFilename)[0]
else:
    outputFilename = options.out
basepath,extension = os.path.splitext(outputFilename)
basename = os.path.basename(basepath)

try:
    os.makedirs(os.path.dirname(outputFilename))
except OSError:
    # Should be okay, or otherwise we will fail later on anyway.
    # The reason to do this instead of checking whether the directory exists, is that the latter can lead to a race condition.
    pass

################################################
# Start doing some actual work

print("Loading %s" % basename)

I = sp.array(Image.open(inputFilename),dtype="float32")/255.0
if I.shape[2]==4:
    I = I[:,:,:3]
elif I.shape[2]!=3:
    parser.error("Only RGB(A) images are supported! (If an alpha channel is present it is ignored.)")

if os.path.islink(outputFilename) or os.path.isfile(outputFilename):
    os.remove(outputFilename)

print("Creating frame")

dgen = space.sample()
D = sp.array(list(itertools.islice(dgen,maxn)),dtype="float32")
invD = la.pinv(D).T
assert D.dtype=="float32" and invD.dtype=="float32"

print("Filtering")

FR = I
for i,d,invd in zip(range(maxn),D,invD):
    print(i, d, invd)
    G = sp.tensordot(I,d,axes=(2,0))
    F = operation(G)
    FR = FR + sp.multiply.outer(F - G,invd)
    assert G.dtype=="float32" and F.dtype=="float32" and FR.dtype=="float32"

    if saveIntermediates:
        FRi = sp.array(sp.clip(sp.round_(255*FR),0,255),dtype='uint8')
        Image.fromarray(FRi).save('%s-%03d%s' % (basepath,i,extension))

if saveIntermediates:
    os.symlink('%s-%03d%s' % (basename,maxn-1,extension), outputFilename)
else:
    FRi = sp.array(sp.clip(sp.round_(255*FR),0,255),dtype='uint8')
    Image.fromarray(FRi).save(outputFilename)

