# Generators for (pseudo) random sequences
import numpy as np

def Halton(n):
    # Each coordinate is associated with a prime.
    primes = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97]
    p = np.array(primes[:n])
    # Initially, every coordinate is set to 1/p
    k = np.ones(n)
    while True:
        # This mirrors the p-ary digits of k behind the decimal point.
        #   So k=[5,5]=[1*2^2+1*2^0, 1*3^1+2*3^0] becomes theta=[1*2^-3+1*2^-1, 1*3^-2+2*3^-1]=[5/8, 7/9].
        theta = 0.0
        km = k
        pr = np.array(p,dtype='float')
        while any(km>0):
            (km,a) = divmod(km,p)
            theta += a/pr
            pr *= p
        yield theta
        k += 1

